/*
 * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
 *                         University Research and Technology
 *                         Corporation.  All rights reserved.
 * Copyright (c) 2004-2005 The University of Tennessee and The University
 *                         of Tennessee Research Foundation.  All rights
 *                         reserved.
 * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2004-2005 The Regents of the University of California.
 *                         All rights reserved.
 * Copyright (c) 2011 Cisco Systems, Inc.  All rights reserved.
 * Copyright (c) 2016      Research Organization for Information Science
 *                         and Technology (RIST). All rights reserved.
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 *
 * This file should be included by any file that needs full
 * version information for the OPAL project
 */

#ifndef OPAL_VERSIONS_H
#define OPAL_VERSIONS_H

#define OPAL_MAJOR_VERSION 4
#define OPAL_MINOR_VERSION 1
#define OPAL_RELEASE_VERSION 9
#define OPAL_GREEK_VERSION "a1"
#define OPAL_WANT_REPO_REV @OPAL_WANT_REPO_REV@
#define OPAL_REPO_REV "v4.1.5-232-gad48c462ff"
#ifdef OPAL_VERSION
/* If we included version.h, we want the real version, not the
   stripped (no-r number) verstion */
#undef OPAL_VERSION
#endif
#define OPAL_VERSION "4.1.9a1"
#define OPAL_CONFIGURE_CLI " \'CC=gcc\' \'CXX=g++\' \'FC=nvfortran\' \'LDFLAGS=-Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/26.1/hpcx-13/291284-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ucx/lib -Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/26.1/hpcx-13/291284-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/hcoll/lib\' \'--with-platform=../contrib/platform/nvhpc/optimized\' \'--enable-mpi1-compatibility\' \'--with-libevent=internal\' \'--without-xpmem\' \'--without-cray-xpmem\' \'--with-slurm\' \'--with-cuda=/proj/cuda/13.0/Linux_x86_64\' \'--with-hcoll=/proj/nv/libraries/Linux_x86_64/26.1/hpcx-13/291284-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/hcoll\' \'--with-ucc=/proj/nv/libraries/Linux_x86_64/26.1/hpcx-13/291284-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ucc\' \'--with-ucx=/proj/nv/libraries/Linux_x86_64/26.1/hpcx-13/291284-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ucx\' \'--prefix=/proj/nv/libraries/Linux_x86_64/26.1/hpcx-13/291284-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ompi\'"

#endif
