#ifndef HCOL_ERRORS_H
#define HCOL_ERRORS_H

#define HCOLL_SUCCESS 0
#define HCOLL_ERROR   -1
#define HCOLL_ERR_OUT_OF_RESOURCE -2
#define HCOLRTE_ERR_RESOURCE_BUSY -4
#define HCOLRTE_ERR_BAD_PARAM -5
#define HCOLL_ERR_NOT_FOUND -13
#define HCOLL_ERR_NOT_SUPPORTED -8
#define HCOLRTE_ERR_NOT_AVAILABLE -16
#define HCOLL_ERR_UNREACH -12
#define HCOLL_ERR_FILE_READ_FAILURE -19
#define HCOLL_ERR_FILE_LOCK_FAILURE -20
#define HCOLL_ERR_FILE_OPEN_FAILURE -21
#define HCOLRTE_ERR_TEMP_OUT_OF_RESOURCE -3
#endif // HCOL_ERRORS_H
